package sun.security.jgss.spnego;

import sun.security.jgss.ProviderList;
import org.ietf.jgss.GSSName;
import sun.security.jgss.SunProvider;
import sun.security.jgss.spi.GSSContextSpi;
import sun.security.jgss.GSSCaller;
import org.ietf.jgss.GSSException;
import sun.security.jgss.spi.GSSCredentialSpi;
import java.util.Vector;
import sun.security.jgss.krb5.Krb5InitCredential;
import sun.security.jgss.krb5.Krb5MechFactory;
import sun.security.jgss.krb5.Krb5AcceptCredential;
import sun.security.jgss.krb5.Krb5NameElement;
import sun.security.jgss.GSSUtil;
import sun.security.jgss.spi.GSSNameSpi;
import sun.security.jgss.GSSManagerImpl;
import org.ietf.jgss.Oid;
import java.security.Provider;
import sun.security.jgss.spi.MechanismFactory;

public final class SpNegoMechFactory implements MechanismFactory
{
    static final Provider PROVIDER;
    static final Oid GSS_SPNEGO_MECH_OID;
    private static Oid[] nameTypes;
    private static final Oid DEFAULT_SPNEGO_MECH_OID;
    final GSSManagerImpl manager;
    final Oid[] availableMechs;
    
    private static SpNegoCredElement getCredFromSubject(GSSNameSpi gssNameSpi, boolean initiate) throws GSSException {
        Vector<GSSCredentialSpi> searchSubject = GSSUtil.searchSubject(gssNameSpi, SpNegoMechFactory.GSS_SPNEGO_MECH_OID, initiate, SpNegoCredElement.class);
        SpNegoCredElement spNegoCredElement = (searchSubject == null || searchSubject.isEmpty()) ? null : ((SpNegoCredElement)searchSubject.firstElement());
        if (spNegoCredElement != null) {
            GSSCredentialSpi internalCred = spNegoCredElement.getInternalCred();
            if (GSSUtil.isKerberosMech(internalCred.getMechanism())) {
                if (initiate) {
                    Krb5MechFactory.checkInitCredPermission((Krb5NameElement)((Krb5InitCredential)internalCred).getName());
                }
                else {
                    Krb5MechFactory.checkAcceptCredPermission((Krb5NameElement)((Krb5AcceptCredential)internalCred).getName(), gssNameSpi);
                }
            }
        }
        return spNegoCredElement;
    }
    
    public SpNegoMechFactory(GSSCaller caller) {
        this.manager = new GSSManagerImpl(caller, false);
        Oid[] mechs = this.manager.getMechs();
        this.availableMechs = new Oid[mechs.length - 1];
        int i = 0;
        int n = 0;
        while (i < mechs.length) {
            if (!mechs[i].equals(SpNegoMechFactory.GSS_SPNEGO_MECH_OID)) {
                this.availableMechs[n++] = mechs[i];
            }
            ++i;
        }
        int j = 0;
        while (j < this.availableMechs.length) {
            if (this.availableMechs[j].equals(SpNegoMechFactory.DEFAULT_SPNEGO_MECH_OID)) {
                if (j != 0) {
                    this.availableMechs[j] = this.availableMechs[0];
                    this.availableMechs[0] = SpNegoMechFactory.DEFAULT_SPNEGO_MECH_OID;
                    break;
                }
                break;
            }
            else {
                ++j;
            }
        }
    }
    
    @Override
    public GSSNameSpi getNameElement(String name, Oid nameType) throws GSSException {
        return this.manager.getNameElement(name, nameType, SpNegoMechFactory.DEFAULT_SPNEGO_MECH_OID);
    }
    
    @Override
    public GSSNameSpi getNameElement(byte[] name, Oid nameType) throws GSSException {
        return this.manager.getNameElement(name, nameType, SpNegoMechFactory.DEFAULT_SPNEGO_MECH_OID);
    }
    
    @Override
    public GSSCredentialSpi getCredentialElement(GSSNameSpi name, int initLifetime, int acceptLifetime, int usage) throws GSSException {
        SpNegoCredElement credFromSubject = getCredFromSubject(name, usage != 2);
        if (credFromSubject == null) {
            credFromSubject = new SpNegoCredElement(this.manager.getCredentialElement(name, initLifetime, acceptLifetime, null, usage));
            return credFromSubject;
        }
        return credFromSubject;
    }
    
    @Override
    public GSSContextSpi getMechanismContext(GSSNameSpi gssNameSpi, GSSCredentialSpi credFromSubject, int n) throws GSSException {
        if (credFromSubject == null) {
            credFromSubject = getCredFromSubject(null, true);
        }
        else if (!(credFromSubject instanceof SpNegoCredElement)) {
            return new SpNegoContext(this, gssNameSpi, new SpNegoCredElement(credFromSubject), n);
        }
        return new SpNegoContext(this, gssNameSpi, credFromSubject, n);
    }
    
    @Override
    public GSSContextSpi getMechanismContext(GSSCredentialSpi credFromSubject) throws GSSException {
        if (credFromSubject == null) {
            credFromSubject = getCredFromSubject(null, false);
        }
        else if (!(credFromSubject instanceof SpNegoCredElement)) {
            return new SpNegoContext(this, new SpNegoCredElement(credFromSubject));
        }
        return new SpNegoContext(this, credFromSubject);
    }
    
    @Override
    public GSSContextSpi getMechanismContext(byte[] array) throws GSSException {
        return new SpNegoContext(this, array);
    }
    
    @Override
    public final Oid getMechanismOid() {
        return SpNegoMechFactory.GSS_SPNEGO_MECH_OID;
    }
    
    @Override
    public Provider getProvider() {
        return SpNegoMechFactory.PROVIDER;
    }
    
    @Override
    public Oid[] getNameTypes() {
        return SpNegoMechFactory.nameTypes;
    }
    
    static {
        PROVIDER = new SunProvider();
        GSS_SPNEGO_MECH_OID = GSSUtil.createOid("1.3.6.1.5.5.2");
        SpNegoMechFactory.nameTypes = new Oid[] { GSSName.NT_USER_NAME, GSSName.NT_HOSTBASED_SERVICE, GSSName.NT_EXPORT_NAME };
        DEFAULT_SPNEGO_MECH_OID = (ProviderList.DEFAULT_MECH_OID.equals(SpNegoMechFactory.GSS_SPNEGO_MECH_OID) ? GSSUtil.GSS_KRB5_MECH_OID : ProviderList.DEFAULT_MECH_OID);
    }
}
